###############################################################################
# UCS:Registered.tcl                          https://nbox.org/eggdrop/scripts/
#
# Copyright (C) 2004 Yann Richard (Ze) - <ze at nbox dot org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License V2 as published by
#   the Free Software Foundation;
# 
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
# 
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   If any clause are declared invalid in your contry only Copyright will
#   be valid and I forbiden the use of my software in theirs contry until
#   i release a new version with a Licence compatible with the GPL.
#
###############################################################################
# 
# This Tcl provide new information about if IRC's users. it provide functions
# to know if someone is registered to X or not.
#
# This script is useful for script's developper.
#
# Note: 
#       Be carefull, when eggdrop just connected and join channels the script
#       could says someone isn't registered whereas he is registered.
#       The script take some time (depends number of channels and users) before
#       initial scanning is done.
#
# Usage:
#
#   UCS:Registered:IsRegistered $nick
#     Return "*" if $nick isn't registered, his username if registered.
#
###############################################################################
# Changes:
#
# 0.2 (30/10/2004) The Halloween Release ;)
# -------------------------------------------------------------------------------
# Foundby     Fixedby     What....
#             Ze          Change script name
#             Ze          Update procedure names
#             Ze          Added UCS(Registered:VERSION)
#             Ze          Changed return value for UCS:Registered:IsRegistered
#             Ze          Added DEBUG facility
# Ze          Ze          fix tcl syntax in USC:OnPlusLa in if condition
# Ze          Ze          fix bad parsing on the RAW 330 event
#             Ze          Don't whois when it's the UCS's bot
#
# 0.1 (13/10/2004) First public release (Registered.tcl)
#
###############################################################################
# TODO:
#     Rename Registered global to UCSRegistered..
###############################################################################

set UCS(Registered:VERSION) "0.2"
set UCS(Registered:DEBUG) 1

###############################################################################
### Event
###############################################################################
bind raw - 330 UCS:Registered:RAW330
proc UCS:Registered:RAW330 {from keyword args} {
  set args [split $args]
  set nick [string tolower [lindex $args 1]]
  set username [lindex $args 2]
  UCS:Registered:Debug "WHOIS: $nick is registered as $username $args"
  # grab information from $args
  USC:Registered:Register $nick $username
}

bind evnt - "disconnect-server" UCS:Registered:OnEggQuit
proc UCS:Registered:OnEggQuit {type} {
  global UCSRegistered
  unset UCSRegistered
}

bind join - * UCS:Registered:OnJoin
proc UCS:Registered:OnJoin {nick userhost handle channel} {
  global botnick

  # 0- Est-ce que c'est l'eggdrop ?
  #
  if { $nick == $botnick } {
    # Il faut rcuperer les informations qui concerne les personnes
    # mais peut etre pas tout de suite...
    UCS:Registered:Debug "I just join $channel going to scan this channel"
    set cmd "UCS:Registered:UpdateChannelUserInfo $channel"
    set t [utimer 10 $cmd]
  }

  # 1- Est il deja enregistr
  #
  set username [UCS:Registered:IsRegistered $nick]
  if { $username != "*" } {
    UCS:Registered:Debug "$nick is already registered as $username"
    return 1
  }

  # 2- Verifier que le user@host n'est pas *.users.undernet.org auquel cas on a
  #    notre rponse
  if { [UCS:Registered:IsUndernetUserhost $userhost] == 1 } {
    set host [lindex [split $userhost "@"] 1]
    set username [lindex [split $host "."] 0]
    USC:Registered:Register $nick $username
  }
  # 3- Verify if it's the Undernet Channel Service Bot
  set nickuserhost "${nick}!${userhost}"
  if { $nickuserhost == "x!cservice@undernet.org" } {
    # its The Channel service bot...
    return 1
  }
  
  # 4- Faire un whois et noter que l'analyse est en cour (avec le timestamps)
  #
  putserv "WHOIS $nick"
}

bind part - "*" UCS:Registered:OnPart
bind kick - "*" UCS:Registered:OnKick
bind sign - "*" UCS:Registered:OnQuit

proc UCS:Registered:OnQuit {nick userhost handle channel reason} {
  USC:OnPlusLa $nick
}
proc UCS:Registered:OnPart {nick userhost handle channel msg} {
  USC:OnPlusLa $nick
}
proc UCS:Registered:OnKick {nick userhost handle channel target reason} {
  USC:OnPlusLa $target
}

# Set user state to 'not authenticated' for the given user
proc USC:OnPlusLa {nick} {
  set ok "0"
  foreach chan [channels] {
    if { [onchan $nick $chan] } {
      # Always on a chan and so already registered
      set ok "1"
    }
  }
  if { ($ok == "O") && ([UCS:Registered:IsRegistered $nick] != "*") } {
    # We don't know now if the same nick rejoin
    # that he always registered
    UCS:Registered:UnRegister $nick
  }
}

###############################################################################
### Public procedure
###############################################################################
# Return "*" if $nick is not registered
# else it return his UnderNet's username
proc UCS:Registered:IsRegistered {nick} {
  global UCSRegistered
  
  if { [info exists UCSRegistered([string tolower $nick])] } {
    return $UCSRegistered([string tolower $nick])
  }
  return "*"
}

###############################################################################
### Private procedure
###############################################################################
proc UCS:Registered:Debug {texte} {
  global UCS
  if { $UCS(Registered:DEBUG) == 1 } {
    putlog "$texte"
  }
}


# Save that $nick is registered as $username
# and if oldnick is set, Unregister $oldnick
proc USC:Registered:Register {nick username {oldnick ""}} {
  global UCSRegistered
  UCS:Registered:Debug "$nick is now registered as $username"
  set UCSRegistered([string tolower $nick]) $username
  if { $oldnick != "" } {
    unset UCSRegistered([string tolower $oldnick])
  }
}

# Unload the registration for $nick
proc UCS:Registered:UnRegister {$nick} {
  global UCSRegistered
  UCS:Registered:Debug "$nick is now not registered"
  unset UCSRegistered([string tolower $nick])
}

# Return 1 if the userhost match an Undernet's authenticated userhost
#        O otherwise
proc UCS:Registered:IsUndernetUserhost {userhost} {
  if { [string match {*.users.undernet.org} $userhost] } {
    return 1
  }
  return 0
}

# Update users state for all eggdrop's channel
proc UCS:Registered:UpdateAllUserInfo {} {
  global UCSRegistered
  UCS:Registered:Debug "Update user info for all channel"
  foreach chan [channels] {
    foreach nick [chanlist $chan] {
      UCS:Registered:UpdateChannelUserInfo $chan
    }
  }
  set UCSRegistered(STATUS) 1
}

# Update user state for the given channel
proc UCS:Registered:UpdateChannelUserInfo {chan} {
  UCS:Registered:Debug "Updating $chan user's info"
  foreach nick [chanlist $chan] {
    set host [getchanhost $nick]
    if { [UCS:Registered:IsUndernetUserhost $host] == 0} {
      # 3- Verify if it's the Undernet Channel Service Bot
      set nickuserhost "${nick}!${host}"
      if { $nickuserhost == "x!cservice@undernet.org" } {
        # its The Channel service bot...
        return 1
      }
      puthelp "WHOIS $nick"
    } else {
      USC:Registered:Register $nick [UCS:Registered:ExtractUsername $host]
    }
  }
}

# Return the undernet's username who are included in the userhost
proc UCS:Registered:ExtractUsername {userhost} {
  return [lindex [split [lindex [split $userhost "@"] 1] "."] 0]
}

# Init the script
proc UCS:Registered:init {} {
  global UCSRegistered
  UCS:Registered:Debug "Initializing UCS:Registered script ....."
    # We are sure that this tcl script his read after start or restart
    # by this mini'hack" we fuck rehash.
    foreach chan [channels] {
      set UCSRegistered(STATUS,$chan) 0
    }
}
###############################################################################
# Script Init
UCS:Registered:init
putlog "Loaded: UCS:Registered v$UCS(Registered:VERSION) - Copyright (C) 2004 - Yann Richard (Ze @ UnderNet)"
### EOF ###
